plot_profile = function(dat_file,cy_file,nchr,sigTh=0.05,peelOff=2,peaks="")
{
  dat=read.delim(dat_file,skip=1,header=TRUE)
  alf=read.delim(dat_file,skip=1,header=TRUE)
  dat =dat[,c(2,4,5,6,10,11,14)]
  for(i in 1:dim(dat)[1])
    {
      if(dat[i,6]<peelOff)
        {
          dat[i,5] = dat[i,7]
          alf[i,10] = alf[i,14]
        }
    }
 
  amp=dat[which(dat[,1]=="Amp"),]
  del=dat[which(dat[,1]=="Del"),]
  cytoband=read.table(cy_file)
  nAxis=5
  scale=1.4
  wd=15
  wdMin=15
  chrLim=matrix(0,nchr,2)      #chromosome limits
  centromere=numeric()  #centromere limits
  cum=0
  for(i in 1:nchr)
    {
      ind=cytoband[,1]==i
      indP=regexpr("p",cytoband[ind,4])>0
      indQ=regexpr("q",cytoband[ind,4])>0
      
      chrLim[i,]=cbind(min(cytoband[ind,2]),max(cytoband[ind,3]))
      if(nchr==22)
        centromere=c(centromere,max(cytoband[ind,3][indP]))
      cum=c(cum,cum[i]+chrLim[i,2])
    }
  
                                        #layout
  par(mar=c(0,8,0,2),cex.lab=3)
  layout(rbind(4,2,1,3,5),10,c(7,9,1,9,7))
  
                                        #plot genome annotation
  plot(NULL,NULL,axes=FALSE,xlim=c(cum[1],cum[nchr+1]),ylim=c(-1.1,1.1),xlab="",ylab="")
  sig=1
  
  for(i in 1:nchr)
    {
      off=cum[i]
      rect(cum[i],0,cum[i+1],sig,col="grey",lwd=2)
      if(nchr==22)
        lines(c(off+centromere[i],off+centromere[i]),c(0,sig),col="red",lwd=3)
      text(off+mean(chrLim[i,]),-sig*0.7,paste(i),cex=1)
      sig=sig*(-1)
    }
  
                                        #scale plots
  ind=amp[,5]!=0
  amp[ind,5]=log10(1-log10(amp[ind,5]))
  
  ind=del[,5]!=0
  del[ind,5]=log10(1-log10(del[ind,5]))
  
  maxQ=max(c(amp[,5],del[,5]))
                                        #plot amplifications
  plot(NULL,NULL,axes=FALSE,xlab="",ylab="FDR q-values",xlim=c(cum[1],cum[nchr+1]),ylim=c(0,maxQ))
  for(i in 1:nchr)
    {
      off=cum[i]
      ind=amp[,1]=="Amp" & amp[,2]==i
      start=amp[ind,3]
      end=amp[ind,4]
      q=amp[ind,5]
      n=length(q)
      x=numeric(2*n+1)
      y=numeric(2*n+1)
      x[2*(1:n)-1]=off+start
      x[2*(1:n)]=off+end
      x[2*n+1]=cum[i+1]
      y[2*(1:n)-1]=q
      y[2*(1:n)]=q
      y[2*n+1]=0
    lines(c(off,x),c(0,y),type="s",col="red",lwd=2)
    }
                                        #plot significance threshold
  lines(cum[c(1,nchr+1)],log10(1-log10(sigTh))*c(1,1),col="green",lwd=2)
                                        #plot y-axis
  expo=round(10^(maxQ/(nAxis-1)*(0:(nAxis-1)))-1)
  axis(2,log10(expo+1),10^(-expo),lwd=3,cex.axis=1.4)
  
                                        #plot deletions
  plot(NULL,NULL,axes=FALSE,xlab="",ylab="FDR q-values",xlim=cum[c(1,nchr+1)],ylim=c(-maxQ,0))
  for(i in 1:nchr)
    {
      off=cum[i]
      ind=del[,1]=="Del" & del[,2]==i
      start=del[ind,3]
      end=del[ind,4]
      q=del[ind,5]
      n=length(q)
      x=numeric(2*n+1)
      y=numeric(2*n+1)
      x[2*(1:n)-1]=off+start
      x[2*(1:n)]=off+end
      x[2*n+1]=cum[i+1]
      y[2*(1:n)-1]=q
      y[2*(1:n)]=q
      y[2*n+1]=0
      lines(c(off,x),-c(0,y),type="s",col="blue",lwd=2)
    }
                                        #plot significance threshold
  lines(cum[c(1,nchr+1)],-log10(1-log10(sigTh))*c(1,1),col="green",lwd=2)
                                        #plot y-axis
  expo=round(10^(maxQ/(nAxis-1)*(0:(nAxis-1)))-1)
  axis(2,-log10(expo+1),10^(-expo),lwd=3,cex.axis=1.4)
  
                                        #gene annotation
                                        #amp
  dx=30*10^6
  plot(NULL,NULL,axes=FALSE,xlim=cum[c(1,nchr+1)],ylim=c(0,2),xlab="",ylab="")
  pos=numeric()
  qval=numeric()
  annot=character()
  for(i in 1:nchr)
    {
      off=cum[i]
      ind=(alf[,"Chromosome"]==i) & (alf[,"Type"]=="Amp") & (alf[,"qValues"]<=sigTh) & (alf[,"Size_Mb"]<wd)
      if(sum(ind)>0)
        {
          pos=c(pos,off+apply(alf[ind,5:6],1,mean))
          annot=c(annot,paste(alf[ind,"Cytoband"]))
          qval=c(qval,paste(alf[ind,"qValues"]))
        }
    }
  selInd=logical(length(pos))
  for(i in 1:length(pos))
    {
      ind=abs(pos[i]-pos) < 10^6*wdMin
      if(sum(ind)>1)
        {
          if(qval[i]==min(qval[ind]))
            selInd[i]=TRUE
          else
            selInd[i]=FALSE
        }
      else
        selInd[i]=TRUE
      
    }
  pos=pos[selInd]
  annot=annot[selInd]
  posTXT=pos
  nShift=0
  if(length(pos) >=2)
    {
      for(j in 2:length(pos))
        {
          if(abs(posTXT[j]-posTXT[j-1])<= dx)
            {
              nShift=nShift+1
              posTXT[j]=posTXT[j]+nShift*dx
            }
          else
            nShift=0
        }
    }
  text(posTXT,rep(1,length(pos)),annot,srt=90,cex=1.3,offset=0)
  for(i in 1:length(pos))
    {
      lines(c(pos[i],pos[i],posTXT[i],posTXT[i]),c(0,0.1,0.2,0.25),lwd=2)
    }
                                        #del
  plot(NULL,NULL,axes=FALSE,xlim=cum[c(1,nchr+1)],ylim=c(0,-2),xlab="",ylab="")
  pos=numeric()
  qval=numeric()
  annot=character()
  for(i in 1:nchr)
    {
      off=cum[i]
      ind=(alf[,"Chromosome"]==i) & (alf[,"Type"]=="Del") & (alf[,"qValues"]<=sigTh) & (alf[,"Size_Mb"]<wd)
      if(sum(ind)>0)
        {
          pos=c(pos,off+apply(alf[ind,5:6],1,mean))
          annot=c(annot,paste(alf[ind,"Cytoband"]))
          qval=c(qval,paste(alf[ind,"qValues"]))
        }
    }
  selInd=logical(length(pos))
  for(i in 1:length(pos))
    {
      ind=abs(pos[i]-pos) < 10^6*wdMin
      if(sum(ind)>1)
        {
          if(qval[i]==min(qval[ind]))
            selInd[i]=TRUE
          else
            selInd[i]=FALSE
        }
      else
        selInd[i]=TRUE
      
    }
  pos=pos[selInd]
  annot=annot[selInd]
  posTXT=pos
  nShift=0
  if(length(pos) >=2)
    {
      for(j in 2:length(pos))
        {
          if(abs(posTXT[j]-posTXT[j-1])<= dx)
            {
              nShift=nShift+1
              posTXT[j]=posTXT[j]+nShift*dx
            }
          else
            nShift=0
        }
    }
  text(posTXT,rep(-1,length(pos)),annot,srt=90,cex=1.3,offset=0)
  for(i in 1:length(pos))
    {
      lines(c(pos[i],pos[i],posTXT[i],posTXT[i]),c(-2,-1.9,-1.8,-1.75),lwd=2)
    }
}
