#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <vector>
#include <list>
#include <math.h>
#include<getopt.h>

#include "installdir.h"

using namespace std;


void qvalues(vector<double> p,vector<double> &q);
void acf(vector<double> dat,vector<double> &acfOut,long k);
void segment(vector<double> dat,vector<long> &segStart,vector<long> &segEnd,double alpha,long minWd);
void expRanks(vector<double> expRaw,vector<long> mapExp,long nExp,vector<long> &expRankVec);
bool isInSeg(vector<long> chr,vector<long> start,vector<long> end,long tChr,long tPos);
void sort_cnv(vector<long> &cnvChr,vector<long> &cnvStart,vector<long> &cnvEnd);

inline void print_header()
{
  string usage_header ="\n\t --- C-GARS: Cancer Genome Analysis by Rank Sums ---\n \
      \t\t    Version 0.5   29/October/2013\n \
      \t     Universiy of Cologne \n \
      \t     Author: Martin Peifer\n\n";
  cerr << usage_header;
};

class mainCLS
{
 private:
  string outName;   //name for output files
  string cnName;    //filename of raw copy numbers
  string matName;   //filename of rank matrix 
  string infoName;  //filename of info file
  string segName;   //filename of segment file
  string exclList;  //filename to exlclude regions from analysis
  string build;     //genome build
  //double uq,lq;     //upper and lower quantile
  vector<double> upperq;
  vector<double> lowerq;
  ofstream logSt;   //stream for loging information
 public:
  //methods of analyze module
  void parse_analyze(int argc,char *argv[]);
  void readRawCN();
  void readSeg();
  void computeRanks();
  void computeNullMax(long n);
  void computeRankSum();
  void segmentRankSum();
  void computeStats();
  void leaveOut();
  void annotateSeg();
  void writeOutput();
  //plotting module
  void plot(int argc,char *argv[]);
  //variables
  bool isCN,isInfo,isMat,isSeg,isXY;  //file status flags
  bool nocnv;
  string dataFormat;
  vector<string> arrayNames;
  vector<string> sampleNames;
  vector<string> probeName;
  vector<string> chromosome;
  vector<string> expSampleNames;
  vector<long>   position,mapExp;
  vector<vector <double> > rawData; //raw data matrix
  vector<vector <long> > rankData;  //rank matrix
  vector<vector <long> > rankDataExp;  //rank matrix expression
  vector<double> rankSumAmp,rankSumDel; //rank sum data
  vector<long> startSegAmp,endSegAmp;
  vector<long> startSegDel,endSegDel;
  vector<double> w;
  vector<long> nRanks; //number of ranks of each sample !=nPos in case of NA's
  vector<double> pAmp,qAmp,pDel,qDel; //p and q values
  vector<double> meanRSAmp,meanRSDel;
  vector<string> annotAmp,annotDel;
  vector<string> cyAmp,cyDel;
  long nSamp; //total number of samples
  long nPos;  //total number of probes/positions
  long nExp;
  long nSegAmp,nSegDel,nSubSegAmp,nSubSegDel; 
  long mwd;
  double alpha;
  //definitions peakf
  double alfq;
  string alfName;
  vector<string> chrAmp,chrDel;
  vector<long> peaksAmp,peaksDel;
  //leave out parameters
  vector<long> n_leave_out_Amp;
  vector<long> n_leave_out_Del;
  vector<double> res_RS_Amp;
  vector<double> res_RS_Del;
  vector<double> res_p_Amp;
  vector<double> res_p_Del;
  vector<double> res_q_Amp;
  vector<double> res_q_Del;
  double sig_max_Amp,mu_max_Amp;
  double sig_min_Del,mu_min_Del;
  vector<double> sig_Amp;
  vector<double> sig_Del;
};
